require 'renderers'

class BaseController

  attr_accessor :renderer

  def initialize(params = {})
    @params = params
    @renderer = ERBRenderer.new(binding)
  end

  def method_missing(m)
    raise "#{self.class}##{m} is not defined"
  end

  protected
  
  # call-seq:
  #   renderer :text => "Hello World"
  #   renderer :controller => "foo", :action => "bar"
  #
  # Called by an action to customize which
  # renderer to use and what parameters to pass it.
  #
  # _hash_ is merge directly into the renderer.
  # If _hash_ contains a key :text then @renderer
  # is set to a new TextRenderer, otherwise it is set
  # a new ERBRenderer.
  def render(hash)
    if hash.has_key?(:text)
      @renderer = TextRenderer.new
    else
      @renderer = ERBRenderer.new(binding)
    end

    @renderer.merge!(hash)
  end

end
