require 'gserver'
require 'stringio'
require File.dirname(__FILE__) + '/handler'

class Request
  attr_accessor :method, :uri, :query

  def initialize(method, uri, query)
    @method = method
    @uri = uri
    @query = query
  end
end

class Response
  attr_accessor :code, :hash, :io

  def initialize(code, hash, io)
    @code = code
    @hash = hash
    @io = io
  end

  def success(&block)
    start(200, &block)
  end

  def not_found(&block)
    start(404, &block)
  end

  def server_error(&block)
    start(500, &block)
  end

  def start(code, &block)
    @code = code
    block.call(@hash, @io)
  end

end

class Server < GServer

  attr_writer :handler

  def serve(io)
   h = io.gets
   if h
     mh = h.match(/([A-Z]+) (\/[^\?]*)(?:\?([^ \t\n]*))? /)
     method = mh[1]
     uri = mh[2]
     query = mh[3]
     request = Request.new(method, uri, query || "")
     response = Response.new(200, Hash.new, StringIO.new)
     @handler.process(request, response)
     io.puts "HTTP/1.1 #{response.code}"
     for k in response.hash.keys
       io.puts "#{k}: #{response.hash[k]}"
     end
     io.puts
     response.io.rewind
     io.write(response.io.read)
   else
     io.puts
     puts "ERROR"
   end
  end
end

if __FILE__ == $0
  server = Server.new(8080)
  server.audit = true
  server.debug = true
  server.handler = Handler.new("welcome")
  server.start
  server.join
end

